from __future__ import division, print_function
import matplotlib.pyplot as plt
import numpy as np
import scipy.linalg
np.set_printoptions(precision=12)
from control import *

nat_freqs = np.array([0.5,0.5,0.5,0.5]);

#Four compartment thermal model with coupling conductivity g and input
#from fixed temperature source next to compartment one.
T = 1
g = 0.1
A = np.matrix([[1-2*g*T,g*T,0,0],[g*T,1-2*g*T,g*T,0],[0,g*T,1-2*g*T,g*T],[0,0,g*T,1-g*T]])
B = np.matrix([[g*T],[0],[0],[0]])
C = np.matrix([0,0,0,1])
D = np.matrix([0])

print("A matrix:")
print(A)
print("B matrix:")
print(B)

# Problem number of states
N = A.shape[0]

#Natural frequency picking
k = acker(A,B,nat_freqs)

#Print k
print("K values")
print(k)

#calculate K_u:
KuInv = C*np.linalg.inv(np.eye(N)-(A-B*k))*B
Ku = np.linalg.inv(KuInv)
print(['Ku=' + str(Ku)])
    
#Form closed loop matrix 
Ac = A-B*k #equivalent to M in notes
Bc = B*Ku
Cc = C
Dc = D
overall=ss(Ac,Bc,Cc,Dc,T)
print("A-Bk");
print(Ac);

#Print closed-loop eigenvalues and estimate simulation interval
print("Eigenvalues and abs")
ec, evecc = scipy.linalg.eig(Ac)
max_n = 100
for p in ec:
    abs_p = np.abs(p)
    print([p, abs_p])
    max_n = max(max_n, 4*np.log(0.1)/np.log(abs_p))

#Simulate the system
print('Simulation Time Interval = '+str(max_n*T))
t = np.arange(0,max_n*T,T)
sig = 20.0*np.ones(np.size(t))
y,t1,x=lsim(overall,sig,t)

#Plot the results
for i in range(N):
    plt.subplot(N+1, 1, i+1)
    plt.plot(x[i]);
    plt.ylabel('State '+str(i+1))

#Plot the input    
command = np.array(Ku*sig[1] - k*x)
plt.subplot(N+1, 1, N+1)
plt.plot(command[0])
plt.ylabel('u')
plt.xlabel('time steps')

#And label the overall plot
plt.suptitle('States and input using Gains from placing natural frequencies');
plt.show()


